/*
 * Copyright (C) 2008 Luke Kenneth Casson Leighton <lkcl@lkcl.net>
 * Copyright (C) 2008 Alp Toker <alp@atoker.com>
 * Copyright (C) 2008 Apple Inc.
 */

#include "config.h"

#include <wtf/GetPtr.h>
#include "Attr.h"
#include "CSSRule.h"
#include "CSSValue.h"
#include "CanvasPixelArray.h"
#include "DOMWindow.h"
#include "Element.h"
#include "Event.h"
#include "GDOMBinding.h"
#include "GDOMObject.h"
#include "HTMLCollection.h"
#include "Node.h"
#include "NodeList.h"
#include "StyleSheet.h"
#include "Text.h"
#include "gdom/GStringConvert.h"
#include "gdom/GdomAttr.h"
#include "gdom/GdomElement.h"
#include "gdom/GdomEventTargetNode.h"
#include "gdom/GdomNodeList.h"
#include "gdom/PrivateGdomAttr.h"
#include "gdom/PrivateGdomDOMObject.h"
#include "gdom/PrivateGdomElement.h"
#include "gdom/PrivateGdomEventTargetNode.h"
#include "gdom/PrivateGdomNodeList.h"
#include "gdom/PrivateIntGdomAttr.h"
#include "gdom/PrivateIntGdomElement.h"
#include "gdom/PrivateIntGdomNodeList.h"
#include "glib-object.h"

namespace WebCore {
    gpointer toGDOM(Node* node);
    gpointer toGDOM(CSSValue* node);
    gpointer toGDOM(CSSRule* node);
    gpointer toGDOM(Element* node);
    gpointer toGDOM(Document* node);
    gpointer toGDOM(HTMLCollection* node);
    //gpointer toGDOM(SVGPathSeg* node);
    gpointer toGDOM(StyleSheet* node);
    gpointer toGDOM(EventTargetNode* node);
    gpointer toGDOM(EventTarget* node);
    gpointer toGDOM(Event* node);
    gpointer toGDOM(CanvasPixelArray* node);
    gpointer toGDOM(DOMWindow* node);
    gpointer toGDOM(Text* node);
} // namespace WebCore

namespace WebCore {
} /* namespace WebCore */
extern "C" {

GDOMElement::GDOMElement(PassRefPtr<WebCore::Element> impl)
    : GDOMEventTargetNode(impl)
{
}

gchar *
gdom_element_get_attribute (GdomElement *thiz, gchar * name)
{
    g_return_val_if_fail (thiz, 0);
    WebCore::Element * item = (gdom_element_get_private(thiz));
    g_return_val_if_fail (item, 0);
    g_return_val_if_fail (name, 0);
    WebCore::String _g_name = WebCore::String::fromUTF8(name);
    gchar * res = _G_GET_UTF8(item->getAttribute(_g_name));
    return res;

}

void
gdom_element_set_attribute (GdomElement *thiz, gchar * name, gchar * value)
{
    g_return_if_fail (thiz);
    WebCore::Element * item = (gdom_element_get_private(thiz));
    g_return_if_fail (item);
    g_return_if_fail (name);
    g_return_if_fail (value);
    WebCore::String _g_name = WebCore::String::fromUTF8(name);
    WebCore::String _g_value = WebCore::String::fromUTF8(value);
    WebCore::ExceptionCode ec = 0;
    item->setAttribute(_g_name, _g_value, ec);

}

void
gdom_element_remove_attribute (GdomElement *thiz, gchar * name)
{
    g_return_if_fail (thiz);
    WebCore::Element * item = (gdom_element_get_private(thiz));
    g_return_if_fail (item);
    g_return_if_fail (name);
    WebCore::String _g_name = WebCore::String::fromUTF8(name);
    WebCore::ExceptionCode ec = 0;
    item->removeAttribute(_g_name, ec);

}

GdomAttr *
gdom_element_get_attribute_node (GdomElement *thiz, gchar * name)
{
    g_return_val_if_fail (thiz, 0);
    WebCore::Element * item = (gdom_element_get_private(thiz));
    g_return_val_if_fail (item, 0);
    g_return_val_if_fail (name, 0);
    WebCore::String _g_name = WebCore::String::fromUTF8(name);
    PassRefPtr<WebCore::Attr> g_res = WTF::getPtr(item->getAttributeNode(_g_name));
    GdomAttr * res = static_cast<GdomAttr *>(WebCore::toGDOM(g_res.get()));
    return res;

}


/* XXX TODO: custom function gdom_element_set_attribute_node */

GdomAttr *
gdom_element_remove_attribute_node (GdomElement *thiz, GdomAttr * old_attr)
{
    g_return_val_if_fail (thiz, 0);
    WebCore::Element * item = (gdom_element_get_private(thiz));
    g_return_val_if_fail (item, 0);
    g_return_val_if_fail (old_attr, 0);
    WebCore::Attr * _g_old_attr = (gdom_attr_get_private(old_attr));
    g_return_val_if_fail (_g_old_attr, 0);
    WebCore::ExceptionCode ec = 0;
    PassRefPtr<WebCore::Attr> g_res = WTF::getPtr(item->removeAttributeNode(_g_old_attr, ec));
    GdomAttr * res = static_cast<GdomAttr *>(WebCore::toGDOM(g_res.get()));
    return res;

}

GdomNodeList *
gdom_element_get_elements_by_tag_name (GdomElement *thiz, gchar * name)
{
    g_return_val_if_fail (thiz, 0);
    WebCore::Element * item = (gdom_element_get_private(thiz));
    g_return_val_if_fail (item, 0);
    g_return_val_if_fail (name, 0);
    WebCore::String _g_name = WebCore::String::fromUTF8(name);
    PassRefPtr<WebCore::NodeList> g_res = WTF::getPtr(item->getElementsByTagName(_g_name));
    GdomNodeList * res = static_cast<GdomNodeList *>(WebCore::toGDOM(g_res.get()));
    return res;

}

gchar *
gdom_element_get_attribute_ns (GdomElement *thiz, gchar * namespace_uri, gchar * local_name)
{
    g_return_val_if_fail (thiz, 0);
    WebCore::Element * item = (gdom_element_get_private(thiz));
    g_return_val_if_fail (item, 0);
    g_return_val_if_fail (namespace_uri, 0);
    g_return_val_if_fail (local_name, 0);
    WebCore::String _g_namespace_uri = WebCore::String::fromUTF8(namespace_uri);
    WebCore::String _g_local_name = WebCore::String::fromUTF8(local_name);
    gchar * res = _G_GET_UTF8(item->getAttributeNS(_g_namespace_uri, _g_local_name));
    return res;

}


/* XXX TODO: custom function gdom_element_set_attribute_ns */

void
gdom_element_remove_attribute_ns (GdomElement *thiz, gchar * namespace_uri, gchar * local_name)
{
    g_return_if_fail (thiz);
    WebCore::Element * item = (gdom_element_get_private(thiz));
    g_return_if_fail (item);
    g_return_if_fail (namespace_uri);
    g_return_if_fail (local_name);
    WebCore::String _g_namespace_uri = WebCore::String::fromUTF8(namespace_uri);
    WebCore::String _g_local_name = WebCore::String::fromUTF8(local_name);
    WebCore::ExceptionCode ec = 0;
    item->removeAttributeNS(_g_namespace_uri, _g_local_name, ec);

}

GdomNodeList *
gdom_element_get_elements_by_tag_name_ns (GdomElement *thiz, gchar * namespace_uri, gchar * local_name)
{
    g_return_val_if_fail (thiz, 0);
    WebCore::Element * item = (gdom_element_get_private(thiz));
    g_return_val_if_fail (item, 0);
    g_return_val_if_fail (namespace_uri, 0);
    g_return_val_if_fail (local_name, 0);
    WebCore::String _g_namespace_uri = WebCore::String::fromUTF8(namespace_uri);
    WebCore::String _g_local_name = WebCore::String::fromUTF8(local_name);
    PassRefPtr<WebCore::NodeList> g_res = WTF::getPtr(item->getElementsByTagNameNS(_g_namespace_uri, _g_local_name));
    GdomNodeList * res = static_cast<GdomNodeList *>(WebCore::toGDOM(g_res.get()));
    return res;

}

GdomAttr *
gdom_element_get_attribute_node_ns (GdomElement *thiz, gchar * namespace_uri, gchar * local_name)
{
    g_return_val_if_fail (thiz, 0);
    WebCore::Element * item = (gdom_element_get_private(thiz));
    g_return_val_if_fail (item, 0);
    g_return_val_if_fail (namespace_uri, 0);
    g_return_val_if_fail (local_name, 0);
    WebCore::String _g_namespace_uri = WebCore::String::fromUTF8(namespace_uri);
    WebCore::String _g_local_name = WebCore::String::fromUTF8(local_name);
    PassRefPtr<WebCore::Attr> g_res = WTF::getPtr(item->getAttributeNodeNS(_g_namespace_uri, _g_local_name));
    GdomAttr * res = static_cast<GdomAttr *>(WebCore::toGDOM(g_res.get()));
    return res;

}


/* XXX TODO: custom function gdom_element_set_attribute_node_ns */

gboolean
gdom_element_has_attribute (GdomElement *thiz, gchar * name)
{
    g_return_val_if_fail (thiz, 0);
    WebCore::Element * item = (gdom_element_get_private(thiz));
    g_return_val_if_fail (item, 0);
    g_return_val_if_fail (name, 0);
    WebCore::String _g_name = WebCore::String::fromUTF8(name);
    gboolean res = item->hasAttribute(_g_name);
    return res;

}

gboolean
gdom_element_has_attribute_ns (GdomElement *thiz, gchar * namespace_uri, gchar * local_name)
{
    g_return_val_if_fail (thiz, 0);
    WebCore::Element * item = (gdom_element_get_private(thiz));
    g_return_val_if_fail (item, 0);
    g_return_val_if_fail (namespace_uri, 0);
    g_return_val_if_fail (local_name, 0);
    WebCore::String _g_namespace_uri = WebCore::String::fromUTF8(namespace_uri);
    WebCore::String _g_local_name = WebCore::String::fromUTF8(local_name);
    gboolean res = item->hasAttributeNS(_g_namespace_uri, _g_local_name);
    return res;

}

void
gdom_element_focus (GdomElement *thiz)
{
    g_return_if_fail (thiz);
    WebCore::Element * item = (gdom_element_get_private(thiz));
    g_return_if_fail (item);
    item->focus();

}

void
gdom_element_blur (GdomElement *thiz)
{
    g_return_if_fail (thiz);
    WebCore::Element * item = (gdom_element_get_private(thiz));
    g_return_if_fail (item);
    item->blur();

}

void
gdom_element_scroll_into_view (GdomElement *thiz, gboolean align_with_top)
{
    g_return_if_fail (thiz);
    WebCore::Element * item = (gdom_element_get_private(thiz));
    g_return_if_fail (item);
    item->scrollIntoView(align_with_top);

}

gboolean
gdom_element_contains (GdomElement *thiz, GdomElement * element)
{
    g_return_val_if_fail (thiz, 0);
    WebCore::Element * item = (gdom_element_get_private(thiz));
    g_return_val_if_fail (item, 0);
    g_return_val_if_fail (element, 0);
    WebCore::Element * _g_element = (gdom_element_get_private(element));
    g_return_val_if_fail (_g_element, 0);
    gboolean res = item->contains(_g_element);
    return res;

}

void
gdom_element_scroll_into_view_if_needed (GdomElement *thiz, gboolean center_if_needed)
{
    g_return_if_fail (thiz);
    WebCore::Element * item = (gdom_element_get_private(thiz));
    g_return_if_fail (item);
    item->scrollIntoViewIfNeeded(center_if_needed);

}

void
gdom_element_scroll_by_lines (GdomElement *thiz, glong lines)
{
    g_return_if_fail (thiz);
    WebCore::Element * item = (gdom_element_get_private(thiz));
    g_return_if_fail (item);
    item->scrollByLines(lines);

}

void
gdom_element_scroll_by_pages (GdomElement *thiz, glong pages)
{
    g_return_if_fail (thiz);
    WebCore::Element * item = (gdom_element_get_private(thiz));
    g_return_if_fail (item);
    item->scrollByPages(pages);

}

GdomNodeList *
gdom_element_get_elements_by_class_name (GdomElement *thiz, gchar * name)
{
    g_return_val_if_fail (thiz, 0);
    WebCore::Element * item = (gdom_element_get_private(thiz));
    g_return_val_if_fail (item, 0);
    g_return_val_if_fail (name, 0);
    WebCore::String _g_name = WebCore::String::fromUTF8(name);
    PassRefPtr<WebCore::NodeList> g_res = WTF::getPtr(item->getElementsByClassName(_g_name));
    GdomNodeList * res = static_cast<GdomNodeList *>(WebCore::toGDOM(g_res.get()));
    return res;

}

GdomElement *
gdom_element_query_selector (GdomElement *thiz, gchar * selectors)
{
    g_return_val_if_fail (thiz, 0);
    WebCore::Element * item = (gdom_element_get_private(thiz));
    g_return_val_if_fail (item, 0);
    g_return_val_if_fail (selectors, 0);
    WebCore::String _g_selectors = WebCore::String::fromUTF8(selectors);
    WebCore::ExceptionCode ec = 0;
    PassRefPtr<WebCore::Element> g_res = WTF::getPtr(item->querySelector(_g_selectors, ec));
    GdomElement * res = static_cast<GdomElement *>(WebCore::toGDOM(g_res.get()));
    return res;

}

GdomNodeList *
gdom_element_query_selector_all (GdomElement *thiz, gchar * selectors)
{
    g_return_val_if_fail (thiz, 0);
    WebCore::Element * item = (gdom_element_get_private(thiz));
    g_return_val_if_fail (item, 0);
    g_return_val_if_fail (selectors, 0);
    WebCore::String _g_selectors = WebCore::String::fromUTF8(selectors);
    WebCore::ExceptionCode ec = 0;
    PassRefPtr<WebCore::NodeList> g_res = WTF::getPtr(item->querySelectorAll(_g_selectors, ec));
    GdomNodeList * res = static_cast<GdomNodeList *>(WebCore::toGDOM(g_res.get()));
    return res;

}

} /* extern "C" */


#include "config.h"
#include "glib-object.h"
#include <webkit/webkitdefines.h>
#include <webkit/webkitprivate.h> 
#include "DOMWindow.h" /* yukk: TODO - make this optional */
#include "Node.h" /* yukk: TODO - make this optional */
#include "NamedNodeMap.h" /* yukk: TODO - make this optional */
#include "CanvasPattern.h" /* yukk: TODO - make this optional */
#include "CanvasGradient.h" /* yukk: TODO - make this optional */
#include "CanvasStyle.h" /* yukk: TODO - make this optional */
#include "CanvasRenderingContext2D.h" /* yukk: TODO - make this optional */
#include <gdom/PrivateGdomDOMObject.h> /* yukk: TODO - make this optional */
#include <gdom/GdomEventTargetNode.h> /* yukk: TODO - make this optional */
#include <gdom/PrivateGdomEventTargetNode.h> /* yukk: TODO - make this optional on EventTarget*/
#include "Element.h"
#include <gdom/GdomElement.h>
#include <gdom/PrivateGdomElement.h>
#include <gdom/PrivateIntGdomElement.h>
#include <gdom/GdomElementprops.h>

#include "gdom/GStringConvert.h"

#include "CanvasPixelArray.h"
#include "Event.h"
#include "Text.h"
#include "CSSValue.h"
#include "CSSRule.h"
#include "HTMLCollection.h"
#include "StyleSheet.h"
#include "Document.h"
#include "webkitmarshal.h"
#include <gdom/GdomEvent.h>



namespace WebCore {
    gpointer toGDOM(Node* node);
    gpointer toGDOM(CSSValue* node);
    gpointer toGDOM(CSSRule* node);
    gpointer toGDOM(Element* node);
    gpointer toGDOM(Document* node);
    gpointer toGDOM(HTMLCollection* node);
    //gpointer toGDOM(SVGPathSeg* node);
    gpointer toGDOM(StyleSheet* node);
    gpointer toGDOM(EventTargetNode* node);
    gpointer toGDOM(EventTarget* node);
    gpointer toGDOM(Event* node);
    gpointer toGDOM(CanvasPixelArray* node);
    gpointer toGDOM(DOMWindow* node);
    gpointer toGDOM(Text* node);
} // namespace WebCore



G_BEGIN_DECLS

G_DEFINE_TYPE(GdomElement, gdom_element, GDOM_TYPE_EVENT_TARGET_NODE);

/*
struct _GdomElementPrivate {
    class WebCore::Element *item;
};

#define GDOM_ELEMENT_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE((obj), GDOM_TYPE_ELEMENT, GdomElementPrivate))
*/
G_END_DECLS
WebCore::Element *
gdom_element_get_private(GdomElement *request)
{
    GDOMElement *res;
    res = static_cast<GDOMElement*>(gdom_dom_object_get_private(GDOM_DOM_OBJECT(request)));
    return static_cast<WebCore::Element*>(res->impl());
}

G_BEGIN_DECLS

enum {
    BROWSER_EVENT,
    LAST_SIGNAL
};

static guint event_signals[LAST_SIGNAL] = { 0, };

enum {
    PROP_0,
    PROP_TAG_NAME,
    PROP_STYLE,
    PROP_OFFSET_LEFT,
    PROP_OFFSET_TOP,
    PROP_OFFSET_WIDTH,
    PROP_OFFSET_HEIGHT,
    PROP_OFFSET_PARENT,
    PROP_CLIENT_LEFT,
    PROP_CLIENT_TOP,
    PROP_CLIENT_WIDTH,
    PROP_CLIENT_HEIGHT,
    PROP_SCROLL_LEFT,
    PROP_SCROLL_TOP,
    PROP_SCROLL_WIDTH,
    PROP_SCROLL_HEIGHT,
    PROP_FIRST_ELEMENT_CHILD,
    PROP_LAST_ELEMENT_CHILD,
    PROP_PREVIOUS_ELEMENT_SIBLING,
    PROP_NEXT_ELEMENT_SIBLING,
    PROP_CHILD_ELEMENT_COUNT,
};


static void gdom_element_finalize(GObject* object)
{
    /*GdomElement* request = GDOM_ELEMENT(object);*/
    /*GdomElementPrivate* priv = request->priv;*/

    /*GObjectClass *parent_class = g_type_class_peek (g_type_peek_parent (GDOM_TYPE_EVENT_TARGET_NODE));*/

    /* Free any memory or close any files */

    /*parent_class-> finalize (object);*/

    G_OBJECT_CLASS(gdom_element_parent_class)->finalize(object);
}

static void gdom_element__set_property_fn(GObject * object, guint prop_id,
                     const GValue * value,
                     GParamSpec * pspec)
{
    GdomElement *self = GDOM_ELEMENT(object);
    WebCore::Element * gself = (gdom_element_get_private(self));
    switch (prop_id) {
     case PROP_SCROLL_LEFT:
    {
         gself->setScrollLeft((g_value_get_long (value)) );
         break;
    }
     case PROP_SCROLL_TOP:
    {
         gself->setScrollTop((g_value_get_long (value)) );
         break;
    }
    default:
        G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
        break;
    }
}


static void gdom_element__get_property_fn(GObject * object, guint prop_id,
                     GValue * value,
                     GParamSpec * pspec)
{
    GdomElement *self = GDOM_ELEMENT(object);
    WebCore::Element * gself = (gdom_element_get_private(self));
    switch (prop_id) {
    case PROP_TAG_NAME:
    {
         g_value_set_string (value, _G_GET_UTF8(gself->tagName()));
         break;
    }
    case PROP_STYLE:
    {
         PassRefPtr<WebCore::CSSStyleDeclaration> ptr = gself->style();
        g_value_set_object (value, WebCore::toGDOM(ptr.get()));
         break;
    }
    case PROP_OFFSET_LEFT:
    {
         g_value_set_long (value, gself->offsetLeft());
         break;
    }
    case PROP_OFFSET_TOP:
    {
         g_value_set_long (value, gself->offsetTop());
         break;
    }
    case PROP_OFFSET_WIDTH:
    {
         g_value_set_long (value, gself->offsetWidth());
         break;
    }
    case PROP_OFFSET_HEIGHT:
    {
         g_value_set_long (value, gself->offsetHeight());
         break;
    }
    case PROP_OFFSET_PARENT:
    {
         PassRefPtr<WebCore::Element> ptr = gself->offsetParent();
        g_value_set_object (value, WebCore::toGDOM(ptr.get()));
         break;
    }
    case PROP_CLIENT_LEFT:
    {
         g_value_set_long (value, gself->clientLeft());
         break;
    }
    case PROP_CLIENT_TOP:
    {
         g_value_set_long (value, gself->clientTop());
         break;
    }
    case PROP_CLIENT_WIDTH:
    {
         g_value_set_long (value, gself->clientWidth());
         break;
    }
    case PROP_CLIENT_HEIGHT:
    {
         g_value_set_long (value, gself->clientHeight());
         break;
    }
    case PROP_SCROLL_LEFT:
    {
         g_value_set_long (value, gself->scrollLeft());
         break;
    }
    case PROP_SCROLL_TOP:
    {
         g_value_set_long (value, gself->scrollTop());
         break;
    }
    case PROP_SCROLL_WIDTH:
    {
         g_value_set_long (value, gself->scrollWidth());
         break;
    }
    case PROP_SCROLL_HEIGHT:
    {
         g_value_set_long (value, gself->scrollHeight());
         break;
    }
    case PROP_FIRST_ELEMENT_CHILD:
    {
         PassRefPtr<WebCore::Element> ptr = gself->firstElementChild();
        g_value_set_object (value, WebCore::toGDOM(ptr.get()));
         break;
    }
    case PROP_LAST_ELEMENT_CHILD:
    {
         PassRefPtr<WebCore::Element> ptr = gself->lastElementChild();
        g_value_set_object (value, WebCore::toGDOM(ptr.get()));
         break;
    }
    case PROP_PREVIOUS_ELEMENT_SIBLING:
    {
         PassRefPtr<WebCore::Element> ptr = gself->previousElementSibling();
        g_value_set_object (value, WebCore::toGDOM(ptr.get()));
         break;
    }
    case PROP_NEXT_ELEMENT_SIBLING:
    {
         PassRefPtr<WebCore::Element> ptr = gself->nextElementSibling();
        g_value_set_object (value, WebCore::toGDOM(ptr.get()));
         break;
    }
    case PROP_CHILD_ELEMENT_COUNT:
    {
         g_value_set_ulong (value, gself->childElementCount());
         break;
    }
    default:
        G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
        break;
    }
}


static void gdom_element_class_init(GdomElementClass* requestClass)
{
    GObjectClass *g_klass = G_OBJECT_CLASS (requestClass);
    g_klass->finalize = gdom_element_finalize;
    g_klass->set_property = gdom_element__set_property_fn;
    g_klass->get_property = gdom_element__get_property_fn;

    g_object_class_install_property (g_klass, PROP_TAG_NAME,
            g_param_spec_string ("tag_name", /* name */
                "element_tag_name", /* short description */
                "read-only  gchar * Element.tag_name", /* longer - could do with some extra doc stuff here */
                 "", /* default */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_STYLE,
            g_param_spec_object ("style", /* name */
                "element_style", /* short description */
                "read-only  GdomCSSStyleDeclaration * Element.style", /* longer - could do with some extra doc stuff here */
                 GDOM_TYPE_CSS_STYLE_DECLARATION, /* gobject type */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_OFFSET_LEFT,
            g_param_spec_long ("offset_left", /* name */
                "element_offset_left", /* short description */
                "read-only  glong Element.offset_left", /* longer - could do with some extra doc stuff here */
                 G_MINLONG,        /* min */
                G_MAXLONG, /* max */
                0,        /* default */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_OFFSET_TOP,
            g_param_spec_long ("offset_top", /* name */
                "element_offset_top", /* short description */
                "read-only  glong Element.offset_top", /* longer - could do with some extra doc stuff here */
                 G_MINLONG,        /* min */
                G_MAXLONG, /* max */
                0,        /* default */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_OFFSET_WIDTH,
            g_param_spec_long ("offset_width", /* name */
                "element_offset_width", /* short description */
                "read-only  glong Element.offset_width", /* longer - could do with some extra doc stuff here */
                 G_MINLONG,        /* min */
                G_MAXLONG, /* max */
                0,        /* default */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_OFFSET_HEIGHT,
            g_param_spec_long ("offset_height", /* name */
                "element_offset_height", /* short description */
                "read-only  glong Element.offset_height", /* longer - could do with some extra doc stuff here */
                 G_MINLONG,        /* min */
                G_MAXLONG, /* max */
                0,        /* default */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_OFFSET_PARENT,
            g_param_spec_object ("offset_parent", /* name */
                "element_offset_parent", /* short description */
                "read-only  GdomElement * Element.offset_parent", /* longer - could do with some extra doc stuff here */
                 GDOM_TYPE_ELEMENT, /* gobject type */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_CLIENT_LEFT,
            g_param_spec_long ("client_left", /* name */
                "element_client_left", /* short description */
                "read-only  glong Element.client_left", /* longer - could do with some extra doc stuff here */
                 G_MINLONG,        /* min */
                G_MAXLONG, /* max */
                0,        /* default */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_CLIENT_TOP,
            g_param_spec_long ("client_top", /* name */
                "element_client_top", /* short description */
                "read-only  glong Element.client_top", /* longer - could do with some extra doc stuff here */
                 G_MINLONG,        /* min */
                G_MAXLONG, /* max */
                0,        /* default */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_CLIENT_WIDTH,
            g_param_spec_long ("client_width", /* name */
                "element_client_width", /* short description */
                "read-only  glong Element.client_width", /* longer - could do with some extra doc stuff here */
                 G_MINLONG,        /* min */
                G_MAXLONG, /* max */
                0,        /* default */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_CLIENT_HEIGHT,
            g_param_spec_long ("client_height", /* name */
                "element_client_height", /* short description */
                "read-only  glong Element.client_height", /* longer - could do with some extra doc stuff here */
                 G_MINLONG,        /* min */
                G_MAXLONG, /* max */
                0,        /* default */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_SCROLL_LEFT,
            g_param_spec_long ("scroll_left", /* name */
                "element_scroll_left", /* short description */
                "read-write  glong Element.scroll_left", /* longer - could do with some extra doc stuff here */
                 G_MINLONG,        /* min */
                G_MAXLONG, /* max */
                0,        /* default */
                 (GParamFlags)G_PARAM_READWRITE));

     g_object_class_install_property (g_klass, PROP_SCROLL_TOP,
            g_param_spec_long ("scroll_top", /* name */
                "element_scroll_top", /* short description */
                "read-write  glong Element.scroll_top", /* longer - could do with some extra doc stuff here */
                 G_MINLONG,        /* min */
                G_MAXLONG, /* max */
                0,        /* default */
                 (GParamFlags)G_PARAM_READWRITE));

     g_object_class_install_property (g_klass, PROP_SCROLL_WIDTH,
            g_param_spec_long ("scroll_width", /* name */
                "element_scroll_width", /* short description */
                "read-only  glong Element.scroll_width", /* longer - could do with some extra doc stuff here */
                 G_MINLONG,        /* min */
                G_MAXLONG, /* max */
                0,        /* default */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_SCROLL_HEIGHT,
            g_param_spec_long ("scroll_height", /* name */
                "element_scroll_height", /* short description */
                "read-only  glong Element.scroll_height", /* longer - could do with some extra doc stuff here */
                 G_MINLONG,        /* min */
                G_MAXLONG, /* max */
                0,        /* default */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_FIRST_ELEMENT_CHILD,
            g_param_spec_object ("first_element_child", /* name */
                "element_first_element_child", /* short description */
                "read-only  GdomElement * Element.first_element_child", /* longer - could do with some extra doc stuff here */
                 GDOM_TYPE_ELEMENT, /* gobject type */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_LAST_ELEMENT_CHILD,
            g_param_spec_object ("last_element_child", /* name */
                "element_last_element_child", /* short description */
                "read-only  GdomElement * Element.last_element_child", /* longer - could do with some extra doc stuff here */
                 GDOM_TYPE_ELEMENT, /* gobject type */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_PREVIOUS_ELEMENT_SIBLING,
            g_param_spec_object ("previous_element_sibling", /* name */
                "element_previous_element_sibling", /* short description */
                "read-only  GdomElement * Element.previous_element_sibling", /* longer - could do with some extra doc stuff here */
                 GDOM_TYPE_ELEMENT, /* gobject type */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_NEXT_ELEMENT_SIBLING,
            g_param_spec_object ("next_element_sibling", /* name */
                "element_next_element_sibling", /* short description */
                "read-only  GdomElement * Element.next_element_sibling", /* longer - could do with some extra doc stuff here */
                 GDOM_TYPE_ELEMENT, /* gobject type */
                 (GParamFlags)G_PARAM_READABLE));

     g_object_class_install_property (g_klass, PROP_CHILD_ELEMENT_COUNT,
            g_param_spec_ulong ("child_element_count", /* name */
                "element_child_element_count", /* short description */
                "read-only  gulong Element.child_element_count", /* longer - could do with some extra doc stuff here */
                 0,        /* min */
                G_MAXULONG, /* max */
                0,        /* default */
                 (GParamFlags)G_PARAM_READABLE));





    /*g_type_class_remove_private(requestClass, sizeof(GdomElementPrivate));*/
}

static void gdom_element_init(GdomElement* request)
{
    /*GdomElementPrivate* priv = GDOM_ELEMENT_GET_PRIVATE(request);
    request->priv = priv;*/
}

} /* extern "C" */

